/***************************************************************************
                          download_record.h  -  description
                             -------------------
    begin                : Wed Jan 22 2003
    copyright            : (C) 2003 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ED2K_GUI_DOWNLOAD_RECORD_H_INCLUDED_
#define _ED2K_GUI_DOWNLOAD_RECORD_H_INCLUDED_

#include <gtk/gtk.h>
#include <time.h>

#include "colors.h"
#include "gapsstruct.h"

typedef struct
{
	guint8		 hash[16];		// file hash

	gchar		*name;			// file name
	gchar		*nojunkname;		// file name without junk in it
	gchar		*type;			// Audio/Video/etc.
	gchar		*format;		// file extension

	gboolean	 cancelled;		// has just been cancelled - waiting for confirmation
	gboolean	 corrupted;		// flag set if we were notified that it has been corrupted
	gboolean	 stats_okay;	// so we know if any DL recs did not get status infos (debug)

	gaps_record	*gaprecord;		// ptr to gaps_record w/ gaplist and pixmaps

	guint		 partnum;		// x in x.part
	guint32		 size;			// file size in bytes
	guint32		 transferred;		// transferred in bytes
	guint		 availability;  	// availability in %
	guint		 sources;		// number of sources (due to core-GUI protocol always <=255)
	guint8		 status;		// status (paused/looking/error etc.)
	gfloat		 speed;			// speed in kB/s
	guint8		 priority;		// low/normal/high

	time_t		 lasttime;		// last time seen downloading (0=never)
	time_t		 lasttime_complete;	// last seen 100% available (0=never)
	time_t		 time_added;		// time added or first time seen (on upgrade)
	gint		 secs_left;		// time left to download (seconds)

	time_t		 lasttime_status_changed;
	time_t		 lasttime_got_gaps;

	ColorSchemeDL	 last_scheme;		// we save these so we can avoid unnecessary redraws
	GdkColor	*last_txtcol;
} downloadRecord ;

enum {
 STATUS_HASHING=0,
 STATUS_QUEUED=1,
 STATUS_LOOKING,
 STATUS_DOWNLOADING,
 STATUS_PAUSED,
 STATUS_IDS,
 STATUS_NOSRCS,
 STATUS_DONE,
 STATUS_HASHING2,
 STATUS_ERRLOADING,
 STATUS_COMPLETING,
 STATUS_COMPLETE,
 STATUS_CORRUPTED,
 STATUS_ERRHASHING,
 STATUS_TRANSFERRING,
 STATUS_RESUMING,
 STATUS_PAUSING,
 STATUS_LAST
} ;	/* RESUMING and PAUSING are GUI status only */



void		 download_calculate_seconds_left_to_complete (downloadRecord *dl);
const gchar	*download_get_time_left_string (downloadRecord *dl);
const gchar	*download_get_status_string (downloadRecord *dl);
const gchar	*download_get_transfered_string (downloadRecord *dl);
const gchar	*download_get_amount_left_string (downloadRecord *dl);
const gchar	*download_get_speed_string (downloadRecord *dl, gboolean showunit);
const gchar	*download_get_priority_string (downloadRecord *dl);
const gchar	*download_get_availability_string (downloadRecord *dl);
const gchar	*download_get_last_seen_complete_string (downloadRecord *dl, gboolean showago);
const gchar	*download_get_last_seen_downloading_string (downloadRecord *dl, gboolean showago);

void       	 download_set_stats (guint16 index,
           	                     guint8 status,
           	                     gfloat speed,
           	                     guint32 trans,
           	                     guint8 avail,
           	                     guint8 sources);




#endif





