/*  filter.c
*
*   filters out certain filenames...
*
*   (c) 2001 Tim-Philipp Muller
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "mainwindow.h"
#include "misc.h"
#include "gui_options.h"
#include "status_page.h"
#include "filter.h"
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

// global variables

char 		**filters = NULL;
unsigned int 	no_of_filters = 0;
//static char 	*filterlistfilename = NULL;

// local vars
static time_t	last_filterlistfile_modtime = 0;
static char 	delimitors[]  = N_("_ ()[]{}.,!*&^%$-=+#@'<>|/\\;:\"0123456789");

// functions

static const gchar *
get_filterlist_filename (void)
{
	return complete_pref_path (N_("gui_filters"), GUI_CONFIG_IGNORE_INSTANCE);
}

int filter_this_filename (const gchar *fn)
{
	static char fnlc[514];	// we don't want to allocate this dynamically because of returns!?
	int i,j;
	char *badword;

	// TODO: make end-of-string also a delimitor ??
	// copy filename and make it lower case
	for (i=0; i<=strlen(fn); i++) fnlc[i] = tolower(fn[i]);

	if (as_word_in_string(fnlc, N_("preteen")))
		return 1;
	if ( (as_word_in_string(fnlc, N_("rape"))) || (as_word_in_string(fnlc, N_("raped"))) )
		return 1;

	// now check for custom filters from gui_filters
	if (!filters) return 0;		// nothing to check

	for (i=0; i<no_of_filters; i++) {
		badword = filters[i];
		for (j=0; j<strlen(badword); j++) badword[j]=tolower(badword[j]);
		if (badword[0]=='*') {			// filter if substring matches word
			if (strstr(fnlc, badword+1)!=NULL) return 1;
		} else {				// else filter if in there as separate word
			if (as_word_in_string(fnlc, badword)) return 1;
		}
	}

	return 0;
}

/* as_word_in_string
*
* checks if string w is in s as word, ie. surrounded by any combination of
* word delimiters, so that if you want to filter out "rape", you don't filter
* out "grapes"...
*
* note: this is case-sensitive!
*
*/

int as_word_in_string (char *s, char*w)
{
	int i,j;
	char *ww = malloc (strlen(w)+3);

	for (i=0; i<strlen(delimitors); i++) {
		for (j=0; j<strlen(delimitors); j++) {
			ww[0] = delimitors[j];
			strcpy (ww+1, w),
			ww[1+strlen(w)] = delimitors[i];
			ww[1+strlen(w)+1] = 0x00;
			if (strstr (s,ww)) {
				free(ww);
				return 1;
			}
		}

		/* word beginning as left delimitor... */
		if (strncmp(s, ww+1, strlen(ww)-1)==0) {
			free(ww);
			return 1;
		}
	}
	if (ww) free (ww);
	return 0;
}

void read_filters (void)
{
	FILE	*ff;
	char	*item = malloc(2048);
	char	*start;

	free_filters ();
	if (!item) return;
	if ( (ff=fopen(get_filterlist_filename(),N_("r")))==NULL) {
		; //fprintf (stderr, "(Harmless:) Couldn't find \"gui_gilters\" filterlist.\nYou can define a filterlist if you wish.\n");
	} else {
		while (!feof(ff)) {
			fscanf (ff, N_("%s\n"), item);
			start = item;
			while (start[0]==' ') start++;		// get rid of leading spaces
			if (start[0]=='#' || start[0]==0x00) continue;		// comment! -> ignore
			no_of_filters++;
			filters = realloc (filters, no_of_filters*sizeof(char *));
			filters[no_of_filters-1] = (char *) malloc(strlen(item)+1);
			strcpy (filters[no_of_filters-1], item);
//#warning
//			DEBUG_1VAR (2,"found filter: '%s'\n", item);
		}
		; //fprintf (stderr, "(Harmless:) Found filterfile. However, filters are not implemented yet.\n");
		fclose (ff);
		last_filterlistfile_modtime = time(NULL);
	}
	if (item) free(item);
}

void write_filters (void)
{
	// TODO: together with a list for that
	;
}

void free_filters (void)
{
	unsigned int i;

	if (filters) {
		for (i=0; i<no_of_filters; i++) {
			if (filters[i]) free (filters[i]);
		}
		free (filters);
		filters=NULL;
		no_of_filters=0;
	}
}

void check_if_filterlistfile_changed (void)
{
	struct stat flstat;
	if (stat (get_filterlist_filename(), &flstat)==0) { // otherwise error or file doesn't exist
	//	printf ("check\nmtime=%d\nlast_modtime=%d\n",flstat.st_mtime, last_filterlistfile_modtime);
		if (flstat.st_mtime > last_filterlistfile_modtime) {
			if (window) status_msg (_("GUI: filterlist file has changed - rereading it.\n"));
			read_filters();
		}
	}
}
