// gaps.c  (ed2k_gui)
//
// all stuff that has to do with the file gaps...
//
// TODO:
// - replace gaps memory alloc+free with g_ routines (in downloads_data.c, too then!)

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "gaps.h"
#include "downloads_page.h"
#include "icons.h"
#include "gui_options.h"
#include "send_command.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <gtk/gtk.h>

#define COLORS_IN_PROGRESSBAR 14
#define SHADES_OF_GREEN 10

unsigned int flag_core_knows_gaps = 0;		// will be set 1 as soon as we got an answer

static GdkPixmap **oldpixmappool = NULL;	// pool of old pixmaps, so we can free them
						// after next refresh
static unsigned int no_of_oldpixmaps = 0;

static char *xpm_color_header_green[COLORS_IN_PROGRESSBAR] = {
N_("w c #E0E0E0"),	// light grey -> don't know (ie. don't have it, but download is paused)
N_("b c #000000"),  // black -> got it
N_("g c #A3A3A3"),  // grey -> asked for
N_("r c #B40000"),  // red -> not available (FF0000 was a bit too bright)
N_("9 c #05440B"),  // darkest green - >50 sources
N_("8 c #06500C"),  // > 20 sources
N_("7 c #086510"),  // 8 sources
N_("6 c #096E11"),
N_("5 c #0A7D13"),
N_("4 c #0B8F16"),
N_("3 c #0C9E19"),
N_("2 c #0eb61c"),
N_("1 c #10c91f"),
N_("0 c #11D922"),  // lightest green - 1 source
};

static char *xpm_color_header_blue[COLORS_IN_PROGRESSBAR] = {
N_("w c #E0E0E0"),	// light grey -> don't know (ie. don't have it, but download is paused)
N_("b c #000000"),  // black -> got it
N_("g c #A3A3A3"),  // grey -> asked for
N_("r c #B40000"),  // red -> not available
N_("9 c #404894"),  // darkest blue, >50 sources	--- these blues suck
N_("8 c #5081AC"),  // >20 sources
N_("7 c #598FBF"),  // 8 sources
N_("6 c #5D95C7"),
N_("5 c #619CD0"),
N_("4 c #72BAD7"),
N_("3 c #76C0DE"),
N_("2 c #7AC7E6"),
N_("1 c #7FCFEF"),
N_("0 c #87DDFF"),  // lightest blue - 1 source
};

static char **xpm_color_header = xpm_color_header_green;

#define COLCHAR_NOT_FOUND	'w'
#define COLCHAR_GOTIT		'b'
#define COLCHAR_NOT_AVAILABLE	'r'
#define COLCHAR_ASKED		'g'
#define COLCHAR_AVAILABLE	'0'
#define COLCHAR_NONE		'w'	// white or transparent, replaces red if dl paused

// functions

void add_pixmap_to_oldpixmappool (GdkPixmap *pm);

void free_pixmaps (downloadRecord *item)
{
	// TODO: memory leak here?
	//       but then... we can't free the pixmaps if they're still in use
	if (!item) return;
	if (!ITEM_GAPRECORD) return;
	if (ITEM_PIXMAP) add_pixmap_to_oldpixmappool (ITEM_PIXMAP);
	if (ITEM_BITMAP) add_pixmap_to_oldpixmappool (ITEM_BITMAP);
	ITEM_PIXMAP  = NULL;
	ITEM_BITMAP  = NULL;
}


// number of gaps is given from beginning
void init_gaplist (downloadRecord *item, guint no_gaps)
{
	guint gaplistsize = (no_gaps+1) * sizeof(struct gap);

	g_return_if_fail (item!=NULL);

	if (item->gaprecord)
	{
		free_pixmaps (item);

		if (ITEM_GAPLIST)
			free(ITEM_GAPLIST);	

		if (ITEM_GAPRECORD)
		{
			memset (ITEM_GAPRECORD, 0x00, sizeof(struct gaps_record));
			free (ITEM_GAPRECORD);
		}
	}

	// we need space for one more gap than specified
	//   => ie. the terminating gap (all fields zero = end marker)
	item->gaprecord = malloc (sizeof(gaps_record));
	g_return_if_fail (item->gaprecord!=NULL);

	ITEM_GAPLIST = (gap *) malloc (gaplistsize);
	g_return_if_fail (ITEM_GAPLIST!=NULL);

	ITEM_PIXMAP  = NULL;
	ITEM_BITMAP  = NULL;
	memset ((void *) ITEM_GAPLIST, 0x00, gaplistsize);
}

void add_gap (downloadRecord *item, unsigned int start, unsigned int end, unsigned int value)
{
	gap		*g = ITEM_GAPLIST;
	guint	 i = 0;

	g_return_if_fail (item!=NULL);
	g_return_if_fail (ITEM_GAPRECORD!=NULL);
	g_return_if_fail (ITEM_GAPLIST!=NULL);

	// assume: we've got enough space for the gaps allocated + space is initialised with zeros

	while ( (g+i)->start>0 || (g+i)->end>0 || (g+i)->value>0 )
	{
		i++;
	}
	(g+i)->start = start;
	(g+i)->end   = end;
	(g+i)->value = value;
}

// get_gap_value_at_pos
//
// returns the value in gaplist corresponding to position percent
//  (on a 0-1 scale from beginning to end)
//
// TODO: now if there's a very small red area between to black areas, it might not show on the bar
//	 unless it's a very wide progress bar. This is not optimal. Implement checks that make sure
//	 that very small gaps between two same gaps are displayed

unsigned int get_gap_value_at_pos (downloadRecord *item, float percent)
{
	gap		*g = ITEM_GAPLIST;
	guint	 i = 0;
	
	if (percent < 0.0 || percent > 1.0 || g == NULL)
		return POS_NOT_IN_LIST;

	while ( (g+i)->start>0 || (g+i)->end>0 || (g+i)->value>0 )
	{
		unsigned int s, e;
		float fs, fe;
		s = (g+i)->start;
		e = (g+i)->end;
		fs = (1.0 * s) / (1.0 * item->size);
		fe = (1.0 * e) / (1.0 * item->size);
		if (percent >= fs && percent < fe)
			return ((g+i)->value);
		i++;
	}
	return POS_NOT_IN_LIST;
}


char get_colchar_from_value (unsigned int val)
{
	if (val==0)
		return (COLCHAR_NOT_AVAILABLE);

	if (val==GAPCOUNTER_FILLED)
		return (COLCHAR_GOTIT);

	if ((val-(val%1000))==GAPCOUNTER_ASKED)	// available + asked for
	{
		// TODO: do we want/need different shades here as well
		return (COLCHAR_ASKED);
	}

	if (val>=GAPCOUNTER_ASKED)
		return (COLCHAR_NOT_FOUND);

	// means _now_ (after previous checks): available
	if (val>50) return (COLCHAR_AVAILABLE + SHADES_OF_GREEN - 1);
	if (val>20) return (COLCHAR_AVAILABLE + SHADES_OF_GREEN - 2);
	if (val>SHADES_OF_GREEN-3) val=SHADES_OF_GREEN-3;
	return (COLCHAR_AVAILABLE+(gchar)val-1);
}

void create_gaplist_pixmap (downloadRecord *item)
{
	guint		width	  = (DL_CLIST->column[8]).width;
	guint		height	  = DL_CLIST->row_height;
	guint		xpm_lines = 1+COLORS_IN_PROGRESSBAR+height;
	gchar 		**tab	  = alloca (xpm_lines*sizeof(char *));
	guint		i, j;
	gchar   	colchar;

	g_return_if_fail (item!=NULL);
	g_return_if_fail (tab!=NULL);

	if (height>10) height = height-5;	// slim e'm down

	xpm_color_header = (option_greenshading==1) ? xpm_color_header_green : xpm_color_header_blue;

	// print header line: "<x> <y> <colors> <character per color>"
	tab[0] = alloca(100); // 100 should do
	g_snprintf (tab[0], 100, N_("%u %u %u 1"), width, height, COLORS_IN_PROGRESSBAR);

	// copy pointers to colour definition lines
	memcpy (tab+1, xpm_color_header+0, COLORS_IN_PROGRESSBAR*sizeof(char *));

	// allocate space for pixmap lines
	for (i=0; i<height; i++)
	{
		tab[1+COLORS_IN_PROGRESSBAR+i] = alloca (width+1);
		g_return_if_fail (tab[1+COLORS_IN_PROGRESSBAR+i]!=NULL);
	}

	// dynamically create the pixmap

	for (i=0; i<=width; i++)
	{
		for (j=0; j<height; j++)
		{
			if (i==width)
				colchar=0x00;	// terminate end of line
			else colchar=get_colchar_from_value (get_gap_value_at_pos(item, (i*1.0)/(width*1.0)));
			// replace red with transparency if download is paused
			// (doesn't look as depressing)
			if (colchar==COLCHAR_NOT_AVAILABLE)
			{
				if (item->status==STATUS_PAUSED || item->status==STATUS_PAUSING)
				{
					colchar = COLCHAR_NONE;
				}
			}
			tab[1+COLORS_IN_PROGRESSBAR+j][i] = colchar;
		}
	}

	// xpm -> pixmap
	ITEM_PIXMAP = gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &ITEM_BITMAP, NULL, tab);
}

void get_gaps_for_visible_downloads (void)
{
	guint row, downloads_num;

	// do nothing if core can't deliver gaps or user doesn't want to see progress bars
	if ((!CORE_IS_GAP_ENABLED) || (option_progressbar==0))
		return;

	downloads_num = downloads_get_num();

	if (downloads_num == 0)
		return;

	for (row=0; row < downloads_num; row++)
	{
		if (option_visibleonly==1)
		{
			if (gtk_clist_row_is_visible (GTK_CLIST(clist_downloads),row)!=GTK_VISIBILITY_NONE)
			{
				downloadRecord *item = (downloadRecord *) gtk_clist_get_row_data (GTK_CLIST(clist_downloads),row);
				if (item)
				{
					// we don't need new gap info if download is paused
					// and we have gap info available
					if (item->status==STATUS_PAUSED && (ITEM_GAPRECORD))
					{
						if (ITEM_GAPLIST) continue;
					}
					send_get_download_gaps (item->hash);
				}
			}
		} else {
			// if we want to update all items, we can just as well avoid all the
			// gtk_clist... calls
			downloadRecord *item = downloads_get_record_by_index(row);
			if (item)
			{
				/* We don't need new gap info if download hasn't changed for quite some
				 * time and we already have gap info available (note: this is only true
				 * if we always update all downloads' gaps and not just the visible ones)
				 * [NOT TRUE. Availability could change as well in the mean time!]
				 * => ignore this idea
				 */

				/* we don't need new gap info if download is paused or idle
				 * and we have gap info available
				 */
				if ((ITEM_GAPRECORD) && (ITEM_GAPLIST))
				{
					if (item->status==STATUS_PAUSED)
					{
						continue;
					}
				}
				send_get_download_gaps (item->hash);
			}
		}
	}
}

unsigned int get_transfered_from_gaps (downloadRecord *item)
{
	gap *g;
	unsigned int trans = 0;
	unsigned int i=0;

	if ((!item) || (!ITEM_GAPRECORD)) return 0;

	g = ITEM_GAPLIST;
	if (!g) return 0;

	while ( (g+i)->start>0 || (g+i)->end>0 || (g+i)->value>0 )
	{
		if ((g+i)->value>=GAPCOUNTER_FILLED)
		{
//			printf ("gap %03u: %10u -> %10u - diff = %10u - trans = %10u\n", i, (g+i)->start, (g+i)->end, (g+i)->end - (g+i)->start, trans);
			trans += (g+i)->end - (g+i)->start;
		}
		i++;
	}
	return trans;
}


/* find_min_num_of_sources_from_gaplist 
 * 
 * Checks the gaplist and returns the minimum number of sources
 * (ie. the max. num of sources having one and the same block of
 *  a file available)
 *
 * Returns min. number of sources or 0 if no gaplist or error
 *
 */

gint
find_min_num_of_sources_from_gaplist (downloadRecord *item)
{
	gap *g;
	guint min_num=0, i=0;

	if ((!CORE_IS_GAP_ENABLED) || (option_progressbar==0))
		return 0;

	g_return_val_if_fail (item!=NULL, -1);

	if (!ITEM_GAPRECORD)
		return 0;

	g = ITEM_GAPLIST;
	if (!g)
		return 0;

	while ( (g+i)->start>0 || (g+i)->end>0 || (g+i)->value>0 )
	{
		guint val = (g+i)->value;

		i++;

		if (val==0) 							/* part not available -> go on		*/
			continue;
		if (val==GAPCOUNTER_FILLED) 			/* we have this -> go on			*/
			continue;
		if ((val-(val%1000))==GAPCOUNTER_ASKED)	/* available + asked for -> go on	*/
			continue;
		if (val>=GAPCOUNTER_ASKED)				/* whatever this check is for		*/
			continue;

		/* means _now_ (after previous checks): available							*/
		min_num = MAX (min_num, val);
	}

	return min_num;
}



// ---- old pixmap pool ----

//static GdkPixmap **oldpixmappool=NULL;	// pool of old pixmaps, so we can free them
						// after next refresh
//static unsigned int no_of_oldpixmaps = 0;

void
add_pixmap_to_oldpixmappool (GdkPixmap *pm)
{
	g_return_if_fail (pm!=NULL);
	no_of_oldpixmaps++;
	oldpixmappool = (GdkPixmap**) realloc(oldpixmappool, no_of_oldpixmaps * sizeof (GdkPixmap*));
	oldpixmappool[no_of_oldpixmaps-1]=pm;
	return;
}

void free_oldpixmaps (void)
{
	GdkPixmap **old_opmp = oldpixmappool;
	guint no_of_old_opm = no_of_oldpixmaps;
	guint row, j, dlnum;
//	guint dummy;

	if ((!oldpixmappool) || (no_of_oldpixmaps==0)) return;

	dlnum = downloads_get_num();

	if (dlnum == 0)
		return;

	oldpixmappool = NULL;
	no_of_oldpixmaps = 0;

//	dummy=0;

	// TODO: do we really need this? I've not seen a _single_ case where not all pixmaps were
	// freed
	for (row=0; row < dlnum; row++)
	{
		GdkPixmap *pm;
		GdkBitmap *bm;
		gtk_clist_get_pixmap (DL_CLIST, row, 8, &pm, &bm);
		for (j=0; j<no_of_old_opm; j++)
		{
			if (old_opmp[j]==pm)
			{
				old_opmp[j]=NULL;	// make sure we skip it when freeing
				add_pixmap_to_oldpixmappool (pm);	// add again for next time
//				dummy++;
			} else {
				if  ((bm) && old_opmp[j]==bm)
				{
					old_opmp[j]=NULL;	// make sure we skip it when freeing
					add_pixmap_to_oldpixmappool (bm); // add again for next time
				}
			}
		}
	}

	// now that we've filtered out the pixmaps + bitmaps which are still in use => free the rest
//	row = 0;
	for (j=0; j<no_of_old_opm; j++)
	{
		if (old_opmp[j])
		{
//			row++;
			gdk_pixmap_unref (old_opmp[j]);
		}
	}
//#warning debug gaps
//	g_print ("free %d pixmaps... (%d old pixmaps not freed)\n", row, dummy);
	if (old_opmp) free(old_opmp);
	return;
}
