/*
*  gui_options.h
*
*  (c) Tim-Philipp Muller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ED2K_GUI_GUI_OPTIONS_INCLUDED_
#define _ED2K_GUI_GUI_OPTIONS_INCLUDED_

#include <gtk/gtk.h>

#define GUI_CONFIG_IGNORE_INSTANCE	FALSE
#define GUI_CONFIG_USE_INSTANCE		TRUE

#define FILELIST_ITEM_VERSION2	0xf0000000	// marker indicating that list is saved in new format

/* options which are set & evaluated by donkey client ***************************/

extern gboolean option_always_connected;	/* flags: 1=true, 0=false */
extern gboolean	option_verbose;
extern gboolean	option_save_corrupted;
extern gboolean option_autoremove;              /* automatically remove dead servers */
extern gboolean	option_allowpm;			/* allow private messages	     */

extern gfloat	option_max_dl_speed;
extern gfloat	option_max_ul_speed;

extern char    option_incoming [260];
extern char    option_temporary[260];
extern char    option_nickname [50];

extern int	option_doorport;
extern int	option_adminport;

extern int		option_max_connections;
extern int		option_line_down_speed;
extern guint32	core_build_date;

/* options evaluated by gui ***************************************************/

extern int 	option_change_message_1_shown;
extern int	option_minfreetempspace;	/* pause all downloads if space in temp is less than this many MB */extern int	option_autoremoveinvalid;
extern int	option_verify_cancels;
extern int	option_verify_exit;
extern int	option_shutdown_core_on_exit;
extern int	option_connect_make_def_values;
extern int	option_spawn_core_on_startup;		// if none is already running
extern int	option_connect_to_core_on_startup;
extern int	option_auto_clear_searchlist;
extern int	option_show_own_files_in_search;
extern int	option_server_rotate;			// 0=off, >0 = how many minutes before disconnect+reconnect
extern int	option_fetch_when_too_few_servers;
extern int	option_clear_slist_when_fetching_new_ones;

extern int 	option_download_ed2k_link_immediately;	// don't add it to search list, start downloading right away
extern int 	option_restart_crashed_core;		// try to respawn core on crash or kill
extern int 	option_disconnect_if_firewalled;	// disconnect if server (wrongly) thinks that I'm firewalled
extern int 	option_color_whole_line_in_downloads;	// does what it says (yuk!)
extern int 	option_nofnjunk;			// in DL list, tidy filenames of junk
extern int	option_keep_extensions;			// keep extensions when tidying up DL list
extern int	option_disable_doubleclick;		// otherwise mousewheel people might have a problem...
extern int	option_show_blacklist_items_grey;	// instead of not showing them at all...
extern int	option_ignore_queue_deadlock;		// don't try to break 'placed on connection queue' deadlocks
extern int	option_nogreen;					// for those people who've got nifty gtk-styles
extern int	option_gnomeicons;				// AXEL: use alternate GNOME icons instead of kde-like ones
extern int	option_slowconnection;			// to minimise transfers on slow connections
extern int	option_noinsuffdspause;			// don't pause those downloads which have 'insuff.diskspace'
											//  when doing a conditional pause all X
extern int	option_clearstatuswindow;		// clear status window every x minutes (0=never)
extern int	option_logtimestamp;			// show timestamps in status window as well
extern int	option_nolinkstolog;			// don't output ed2k-links to statuslog
extern int	option_progressbar;				// colour progressbar as 'availability'?
extern int	option_greenshading;			// shade available parts in green (no=blue)
extern int	option_blacklistondl;			// add items to download automatically to blacklist
extern int	option_dltransinpercent;		// show 'transfered' in downloads in %
extern int	option_visibleonly;				// progressbar update: update only gaps for visible rows
extern int	option_noactivedlcancel;		// don't allow dl cancels w/o confirmation if dl is not paused
extern int	option_no_app_icon;				// don't set the application icon (set 1 per -i cmd switch)

extern int	option_DLUnselectAfterAction;	// unselect all items after each action
extern int	option_DLSingleSelectionMode;	// put download clist in single selection mode
extern int	option_DLShowPartFileName;		// show the x.part filename in the downloadlist with the filename

extern int	option_SRVUnselectAfterAction;		// unselect all servers after each action
extern int	option_SRVSingleSelectionMode;		// put server clist in single selection mode

extern gint	option_search_notebook_tabs_pos;	// the position of the search notebook tabs
extern gint	option_main_notebook_tabs_pos;		// the position of the main notebook tabs
extern gint	option_main_notebook_tabs_hide_text;

extern guint option_linkhandler_what_to_do_if_no_GUI_running;

extern unsigned int	option_gui_instance;		// set via the command line option '-n'

// KEEP THIS FOR REFERENCE PURPOSES FOR NOW:
//
//extern char    option_defaulthost[128];
//extern int     option_defaultport;
//extern char    option_defaultuser[128];
//extern char    option_defaultpass[128];
//extern char    option_defaultdonkeypath[512];
//extern char	option_serverlist_URL[512];
//extern char	option_serverlist_URL2[512];
//extern char	option_exec_on_complete[513];

extern char	* option_defaulthost;		// host where the donkey client/core runs
extern int	  option_defaultport;
extern char	* option_defaultuser;		// default core login username to show in connect dialog
extern char	* option_defaultpass;		// default core login password to show in connect dialog
extern char	* option_defaultdonkeypath;	// default path to donkey binary
extern char	* option_serverlist_URL;	// complete URL to serverlist html page with ed2k:// links
extern char	* option_serverlist_URL2;	// complete URL to serverlist html page with ed2k:// links
extern char	* option_exec_on_complete;	// shell command to execute when a download is complete; %F=filename

extern gchar	*option_iconthemedir;	/* directory where the icons to use are */

extern guint	 option_maxlogfilesize;			/* max. logfile size in kB */

extern guint	 option_show_extra_search_options;

extern char    clienthost[513];            // actual host used (from connect dialog)
extern int     clientport;                 // actual port used (from connect dialog)

extern int     connect_right_away;         // using default options (don't bring up dialog at start

extern char	* prefpath;			// the prefpath used by complete_pref_path()

extern char STANDARD_SERVERLIST_1[];
extern char STANDARD_SERVERLIST_2[];

/* window properties */

/* set these values generous, so we don't have too much incompatibility
*  between options files when adding new features
*/

#define MAX_NO_OF_CLISTS	16
#define MAX_NO_OF_COLS 		16

enum { DL_COLS, UL_COLS, Q_COLS, SEARCH_COLS, SERVER_COLS, SHARED_COLS, FSHARED_COLS, ULSTATS_COLS };
enum { DL_LIST, UL_LIST, Q_LIST, SEARCH_LIST, SERVER_LIST, SHARED_LIST, FSHARED_LIST, ULSTATS_LIST };

extern int	column_widths [MAX_NO_OF_CLISTS][MAX_NO_OF_COLS];
extern int	lists_sorttype [MAX_NO_OF_CLISTS]; 	// ascending / descending
extern int	lists_sortcol  [MAX_NO_OF_CLISTS]; 	// column to sort by
extern int	dlcols_visibility  [MAX_NO_OF_COLS];	// which download list colums are visible?


#define downloadlist_sortcolumn lists_sortcol[DL_LIST]
#define uploadlist_sortcolumn lists_sortcol[UL_LIST]
#define searchlist_sortcolumn lists_sortcol[SEARCH_LIST]
#define serverlist_sortcolumn lists_sortcol[SERVER_LIST]
#define ulstatslist_sortcolumn lists_sortcol[ULSTATS_LIST]

#define downloadlist_sorttype lists_sorttype[DL_LIST]
#define uploadlist_sorttype lists_sorttype[UL_LIST]
#define searchlist_sorttype lists_sorttype[SEARCH_LIST]
#define serverlist_sorttype lists_sorttype[SERVER_LIST]
#define ulstatslist_sorttype lists_sorttype[ULSTATS_LIST]

#define MAX_NO_OF_PANED_WINS 16

enum { UL_PANE_POS_DEPRECATED, SHARED_PANE_POS, UL_PANE2_POS };

extern int  paned_win_position [MAX_NO_OF_PANED_WINS];

/* these values are necessary because we read the position of the **GDK** window
*  and so get the position of the top left gdk window corner and not of the
*  top left corner of the titlebar
*
*  but we later set the position of the **GTK** window, ie. the position of the
*  top left corner of the titlebar...
*
*  these may vary of course, but they work nicely for me :)
*/

#define X_POS_CORRECT 4
#define Y_POS_CORRECT 28

extern GtkAllocation	main_window_size;


/* ----- functions ----- */

const	gchar	*complete_pref_path (const gchar *fn, gboolean flag_use_instance);

int		check_for_unwanted_URL (char **urlstr);

void write_gui_uploadstats (void);
void uploadstats_read_from_file (void);

void read_gui_options (void);
void write_gui_options (void);
void set_clist_columns (GtkCList *clist, int list_no, int no_of_cols);
void remember_clist_columns (GtkCList *clist, int list_no);

#endif
