/***************************************************************************
                          ramfile.h  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ed2k_gui_ramfile_h_included_
#define _ed2k_gui_ramfile_h_included_

#define GNET_EXPERIMENTAL
#include <gnet/gnet.h>

struct _ramfile
{
	guint		size;		// current size
	guint		pos;		// current position
	GByteArray	*data;		// data buffer
} ;

typedef struct _ramfile ramfile;

enum
{
RAMFILE_SEEK_FROM_HERE=1,
RAMFILE_SEEK_FROM_START,
RAMFILE_SEEK_FROM_END
} ;

extern ramfile	*ramfile_now_active;

// functions

ramfile		*ramfile_new (guint len);
ramfile		*ramfile_new_from_file (const gchar *filename);
void	 	 ramfile_free (ramfile *rf);
guint8	 	 ramfile_read8  (ramfile *rf);
guint16	 	 ramfile_read16 (ramfile *rf);
guint32	 	 ramfile_read32 (ramfile *rf);
gfloat		 ramfile_readfloat (ramfile *rf);
guint8		*ramfile_read (ramfile *rf, guint8 *buf, gint len);
guint8		*ramfile_peek (ramfile *rf, guint8 *buf, gint pos, gint len);
void	 	 ramfile_write8  (ramfile *rf, const gint8 val);
void	 	 ramfile_write16 (ramfile *rf, const gint16 val);
void	 	 ramfile_write32 (ramfile *rf, const gint32 val);
void		 ramfile_writefloat (ramfile *rf, gfloat val);
void	 	 ramfile_write   (ramfile *rf, guint8 *buf, gint len);
void	 	 ramfile_write_i4_string (ramfile *rf, const gchar *string);
gchar		*ramfile_read_i4_string (ramfile *rf);
gboolean  	 ramfile_seek (ramfile *rf, gint inc, gint seektype);
void	 	 ramfile_dump (ramfile *rf, const gchar *infostring);

gboolean	 ramfile_write_to_file (ramfile *rf, const gchar *fn);

void	 	 ramfile_set_active (ramfile *rf);
void	 	 ramfile_unset_active (ramfile *rf);

#define read_8()		ramfile_read8(ramfile_now_active)
#define read_16()		ramfile_read16(ramfile_now_active)
#define read_32()		ramfile_read32(ramfile_now_active)
#define read_float()		ramfile_readfloat(ramfile_now_active)
#define read_IP()		GINT32_TO_LE(ramfile_read32(ramfile_now_active))
#define read_x(a)		ramfile_read(ramfile_now_active,NULL,a)
#define write_8(a)		ramfile_write8(ramfile_now_active,a)
#define write_16(a)		ramfile_write16(ramfile_now_active,a)
#define write_32(a)		ramfile_write32(ramfile_now_active,a)
#define write_IP(a)		ramfile_write32(ramfile_now_active,GINT32_FROM_LE(a))
#define write_float(a)	ramfile_writefloat(ramfile_now_active,a);
#define write_x(buf,len)	ramfile_write(ramfile_now_active,buf,len)
#define	seek(inc,type)		ramfile_seek(ramfile_now_active,inc,type)
#define	read_i4str()		ramfile_read_i4_string(ramfile_now_active)
#define write_i4str(s)		ramfile_write_i4_string (ramfile_now_active,s)

// the ip is already in network byte order - we don't want it to be changed to
//  some other order by ramfile_write32 if platform is not little endian
//
#define ramfile_writeIP(rf,ip)	ramfile_write32(rf,GINT32_FROM_LE(ip))
#define ramfile_readIP(rf)		GINT32_TO_LE(ramfile_read32(rf))
#define ramfile_get_size(rf)	rf->size
#define ramfile_get_pos(rf)	rf->pos
#define ramfile_set_pos(rf,newpos)	rf->pos=newpos
#define ramfile_get_buffer(rf)	(rf->data)->data
#endif

