/***************************************************************************
                          servers_data.h  -  description
                             -------------------
    begin                : Wed Jul 3 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ed2k_gui_servers_data_included_
#define _ed2k_gui_servers_data_included_

#define GNET_EXPERIMENTAL
#include <gnet/gnet.h>
#include <time.h>

typedef enum
{
 SERVERS_CONNECT_STATUS_CONNECTING = 0,
 SERVERS_CONNECT_STATUS_PLACEDONQ,
 SERVERS_CONNECT_STATUS_CANTCONNECT,		// ie. an undesired disconnect
 SERVERS_CONNECT_STATUS_DISCONNECTED,		// ie. a 'normal' disconnected
 SERVERS_CONNECT_STATUS_PLACEDONQ_DEADLOCK,	// oops, deadlocked ?!
 SERVERS_CONNECT_STATUS_DO_SERVER_ROTATE,
 SERVERS_CONNECT_STATUS_THINKS_WE_ARE_FIREWALLED,
 SERVERS_CONNECT_STATUS_INVALID,
} serversConnectStatus;

struct serverRecord
{
	gchar		*name, *desc;
	guint32		 ip, users, files, ping;
	guint16		 port;
	gint8		 pref;
	gboolean	 remove;
	gboolean	 nogood;	// server has rejected us (out of date) or thinks we are firewalled
	time_t		 last_seen_alive;
	time_t		 last_connected;	/* last successfully connected	*/
	time_t		 last_connecting;	/* last tried to connect		*/
} ;

typedef struct serverRecord serverRecord;


// An array of pointers to serverRecord items (note: not necessarily in the same order
//	that the command line client has them in its list
extern GPtrArray	*servers;

// What do we do if the user has selected multiple servers and clicks 'connect?'
//	we need to do the connection process on our own. This is a list of
//	GInetAddr structures we want to connect to
extern GList			*connectlist;


serverRecord	*servers_find_from_address (guint32 ip, guint16 port);
void		 servers_show_dead_alive_stats (void);
void		 servers_clear_connect_to_list (void);
gboolean	 servers_connect_to_next_selected (void);
void		 servers_save_alwaysconnected_option ();
void		 servers_reinstate_alwaysconnected_option();

void		 servers_init(void);
void		 servers_cleanup(void);

void		 servers_mark_all_for_removal (void);
void		 servers_remove_all_old_records (void);
gboolean	 servers_remove_record (serverRecord *srv);
void		 servers_add_or_update_record (guint32 ip, guint16 port, const gchar *name, const gchar *desc, guint32 users, guint32 files, guint32 ping, guint8 pref);

void		 servers_register_connect_status (const gchar *ipOrName, serversConnectStatus status);

#endif


