/***************************************************************************
                          uploadstats_data.c  -  description
                             -------------------
    begin                : Fri Jul 26 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "uploads_data.h"
#include "uploads_page.h"
#include "uploadstats_data.h"

GPtrArray	*uploadstats = NULL;


static void	uploadstats_free_record (uploadstatsRecord *ulstatsr);

// uploadstats_add_or_increase_record
//
// increases the number of uploaded bytes for the upload stats
//	record matching the given hash

void
uploadstats_add_or_increase_record (const uploadRecord *ul, guint timediff)
{
	uploadstatsRecord	*ulstatr = NULL;
	gfloat				 ul_add;

	g_return_if_fail (ul!=NULL);

	if (uploadstats)
		ulstatr = uploadstats_find_from_hash(ul->hash);

	if (!ulstatr)
	{
//		static gint count=0;
		ulstatr = g_new0 (uploadstatsRecord, 1);
		g_return_if_fail (ulstatr!=NULL);
		memcpy (ulstatr->hash, ul->hash, 16);
		ulstatr->namequark = ul->namequark;
		ulstatr->size = ul->size;
		ulstatr->uploaded = 0.0;
		if (!uploadstats)
		{
			uploadstats = g_ptr_array_new();
			g_return_if_fail (uploadstats!=NULL);
		}
		g_ptr_array_add (uploadstats, (gpointer)ulstatr);
//		count++;
//		g_print ("new record #%u\n", count);
	}

	ul_add = ul->speed*(timediff*1.0);
	/* more than ca. 330kB/s for one single upload? unlikely... */
	if (ul_add > 1024.0)
	{
		g_printerr (N_("GUI DEBUG: ul_add = %.1f kB in %s (ul->speed=%f, timediff=%u)\n"),
			ul_add, __FUNCTION__, ul->speed, timediff);
	} else ulstatr->uploaded += ul_add;

	// this condition is here so we can add records when we
	//	read in the upload stats list from file without setting
	//	all values to now...
	if (ul->speed>0.0)
		ulstatr->last_uploaded = time(NULL);
}



// uploadstats_find_from_hash
//
// Finds an uploadstats record from a given file hash
// Returns record or NULL if not found

uploadstatsRecord *
uploadstats_find_from_hash (const guint8 *filehash)
{
	g_return_val_if_fail (filehash!=NULL, NULL);
	if (uploadstats)
	{
		PROCESS_GPTRARRAY ( uploadstats, uploadstatsRecord,
			if ((item) && memcmp(item->hash, filehash, 16)==0) return item;
		);
	}
	return NULL;
}


// uploadstats_free_record
//
// frees an upload stats list record

static void
uploadstats_free_record (uploadstatsRecord *ulstatsr)
{
	g_return_if_fail (ulstatsr!=NULL);
	memset(ulstatsr,0x00,sizeof(ulstatsr));
	g_free(ulstatsr);
}


// uploadstats_remove_record
//
// removes an upload stats record from the list and the clist, and frees it

void
uploadstats_remove_record (uploadstatsRecord *ulstatsr)
{
	g_return_if_fail (ulstatsr!=NULL);
	if (uploadstats)
		g_ptr_array_remove_fast (uploadstats, (gpointer)ulstatsr);
	uploadstatsclist_remove_record (ulstatsr);
	uploadstats_free_record(ulstatsr);
}

// uploadstats_cleanup
//
// clears the whole upload stats list

void
uploadstats_cleanup (void)
{
	if (uploadstats)
	{
		PROCESS_GPTRARRAY_FROM_END ( uploadstats, uploadstatsRecord,
			if (item) uploadstats_remove_record (item);
		);
		g_ptr_array_free(uploadstats, TRUE);
		uploadstats = NULL;
	}
}

